#    Copyright 2004 Andrew Wilkinson <aw@cs.york.ac.uk>.
#
#    This file is part of PyLinda (http://www-users.cs.york.ac.uk/~aw/pylinda)
#
#    PyLinda is free software; you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation; either version 2.1 of the License, or
#    (at your option) any later version.
#
#    PyLinda is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with PyLinda; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import socket

import utils

counter = utils.Counter()

try:
    socket.AF_UNIX
except AttributeError:
    raise ImportError

class Socket:
    def __init__(self):
        self.closed = True

    def create(self):
        i = 0
        while True:
            self.s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            self.addr = "/tmp/pylinda.%i" % (counter(), )
            try:
                self.s.bind(self.addr)
            except socket.error:
                if i > 100:
                    raise
                else:
                    i = i + 1
            else:
                break

        self.s.listen(1)

    def close(self):
        self.send("close_connection")
        self.closed = True

        # wait so all the connections can be correctly closed
        import time
        time.sleep(0.1)

    def accept(self):
        self.s = self.s.accept()[0]
        self.closed = False

    def connect(self, addr):
        self.s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.s.connect(addr)

        self.addr = addr
        self.closed = False

    def getaddr(self):
        return self.addr

    def recv(self, max_msg_size):
        if self.closed:
            return ""
        r = self.s.recv(max_msg_size)
        if r == "close_connection":
            try:
                os.unlink(self.addr)
            except OSError:
                pass
            return ""
        else:
            return r

    def send(self, data, opts = 0):
        if self.closed:
            return

        try:
            self.s.send(data, opts)
        except socket.error:
            # the connection has been broken...
            try:
                os.unlink(self.addr)
            except OSError:
                pass
            self.closed = True
            return ""

    def sendall(self, data, opts = 0):
        self.s.sendall(data, opts)
