#    Copyright 2004 Andrew Wilkinson <aw@cs.york.ac.uk>.
#
#    This file is part of PyLinda (http://www-users.cs.york.ac.uk/~aw/pylinda)
#
#    PyLinda is free software; you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation; either version 2.1 of the License, or
#    (at your option) any later version.
#
#    PyLinda is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with PyLinda; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import struct
import sys
import time
import thread

import shm
import utils

class Socket:
    def __init__(self):
        self.closed = True
        self.create_flag = False

    def __del__(self):
        if not self.closed:
            self.close()

    def create(self, size):
        self.create_flag = True
        self.size = size

        counter = utils.Counter()
        counter.setLimit(255)

        try:
            while True:
               try:
                   self.s1 = shm.create_memory(shm.ftok(sys.argv[0], counter()), size+2)
                   break
               except shm.error:
                   pass
            while True:
               try:
                   self.s2 = shm.create_memory(shm.ftok(sys.argv[0], counter()), size+2)
                   break
               except shm.error:
                   pass
            while True:
               try:
                   self.sem1 = shm.create_semaphore(shm.ftok(sys.argv[0], counter()))
                   break
               except shm.error:
                   pass
            while True:
               try:
                   self.sem2 = shm.create_semaphore(shm.ftok(sys.argv[0], counter()))
                   break
               except shm.error:
                   pass
        except ValueError:
            return False

        self.sem1.setval(0)
        self.sem2.setval(0)

        self.addr = (self.s1.shmid, self.s2.shmid, self.sem1.semid, self.sem2.semid)

        self.s1.attach()
        self.s2.attach()

        self.closed = False

        return True

    def connect(self, addr, size):
        self.size = size
        self.addr = addr

        self.s2 = shm.memory(addr[0])
        self.sem2 = shm.semaphore(addr[2])
        self.s1 = shm.memory(addr[1])
        self.sem1 = shm.semaphore(addr[3])
        self.s1.attach()
        self.s2.attach()

        self.closed = False

    def close(self):
        if not self.closed:
            self.s1.detach()
            self.s2.detach()

            try:
                self.s1.nattch # for some reason this is 1 not 2 as I would expect so the memory
                self.s2.nattch # blocks get freed too early...
            except shm.error:
                pass
            else:
                if self.s1.nattch == 0:
                    shm.remove_memory(self.s1.shmid)
                if self.s2.nattch == 0:
                    shm.remove_memory(self.s2.shmid)
                    
            if self.create_flag:
                shm.remove_semaphore(self.sem1.semid)
                shm.remove_semaphore(self.sem2.semid)

            self.closed = True

    def getaddr(self):
        return self.addr, self.size

    def recv(self, max_msg_size):
        #print "recv"
        self.sem2.P()
        try:
            data = self.s2.read(self.size+2,0)
            size = struct.unpack("=H", data[:2])[0]
            return data[2:2+size]
        finally:
            pass

    def send(self, data, opts=0):
        if len(data) <= self.size:
            self.sem1.Z()
            self.s1.write(struct.pack("=H", len(data))+data,0)
            self.sem1.V()
        else:
            i = 0
            while len(data) - i > self.size:
                self.sem1.Z()
                self.s1.write(struct.pack("=H", self.size)+data[i:i+self.size],0)
                self.sem1.V()
                i += self.size
            self.sem1.Z()
            self.s1.write(struct.pack("=H", len(data[i:i+self.size]))+data[i:i+self.size],0)
            self.sem1.V()

    def sendall(self, data, opts=0):
        self.send(data)
